/*                                                                            */

/*----------------------------------------------------------------------------*/
/* System includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>

/*----------------------------------------------------------------------------*/
/* Main program                                                               */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct RDArgs *rdargs;
	struct {
		char *path;
		} args = {""};
	BPTR lock;

	rdargs = ReadArgs ("PATH",(void *)&args,NULL);
	if (!rdargs)
		{
		PrintFault (IoErr(),NULL);
		return (RETURN_ERROR);
		}

	if (lock = Lock (args.path,SHARED_LOCK))
		{
		struct FileInfoBlock *fib;
		if (fib = AllocVec (sizeof(struct FileInfoBlock),MEMF_CLEAR))
			{
			if (Examine (lock,fib))
				{
				if (fib->fib_DirEntryType > 0)
					{
					char buffer[256];

					while (ExNext (lock,fib))
						{
						switch (fib->fib_DirEntryType)
							{
						case ST_FILE:
							Printf ("%-24s file\n",fib->fib_FileName);
							break;
						case ST_ROOT:
						case ST_USERDIR:
							Printf ("%-24s directory\n",fib->fib_FileName);
							break;
						case ST_SOFTLINK:
							buffer[0] = 0;
							ReadLink (((struct FileLock *)BADDR(lock))->fl_Task,lock,fib->fib_FileName,buffer,256);
							Printf ("%-24s softlink -> %s\n",fib->fib_FileName,buffer);
							break;
						case ST_LINKDIR:
						case ST_LINKFILE:
							{
							BPTR link,oldcd;
							buffer[0] = 0;
							oldcd = CurrentDir (lock);
							if (link = Lock (fib->fib_FileName,SHARED_LOCK))
								{
								NameFromLock (link,buffer,256);
								UnLock (link);
								}
							CurrentDir (oldcd);
							Printf ("%-24s hardlink -> %s\n",fib->fib_FileName,buffer);
							break;
							}
						default:
							Printf ("%-24s unknown (%ld)\n",fib->fib_FileName,fib->fib_DirEntryType);
							}
						}
					}
				else
					{
					SetIoErr (ERROR_OBJECT_WRONG_TYPE);
					PrintFault (IoErr(),args.path);
					}
				}
			else
				PrintFault (IoErr(),args.path);

			FreeVec (fib);
			}
		else
			{
			SetIoErr (ERROR_NO_FREE_STORE);
			PrintFault (IoErr(),args.path);
			}
		UnLock (lock);
		}
	else
		PrintFault (IoErr(),args.path);

	FreeArgs (rdargs);
	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/* End of source text                                                         */
/*----------------------------------------------------------------------------*/
